/*******************************************/
/*** Wham-O-Meter 1.2                    ***/
/*** ----------------                    ***/
/*** Autor: Christoph Sllner            ***/
/*** Webseite: www.mcseven.me            ***/
/*** Lizenz: Creative-Commons            ***/
/***         by-nc-sa (3.0)              ***/
/*******************************************/

#define  F_CPU 10000000

/*******************************************/
/*** Globale Header-Files                ***/
/*******************************************/
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

// Sofware-Single-Wire-UART einbinden
#include "single_wire_UART.h"

// Fonts und Images einbetten
#include "font_arialbig.h"
#include "font_sansserif7regularnumbers.h"
#include "font_tahoma10regular.h"
#include "font_tahoma10bold.h"
#include "image_goal.h"
#include "image_victory.h"
#include "image_defeat.h"
#include "flashStrings.h"
#include "glcd.h"

/*******************************************/
/*** Globale Defines                     ***/
/*******************************************/
// Tasten
#define UPBTN !(PINB&0b00000010)
#define DNBTN !(PINB&0b00000001)
#define OKBTN !(PINB&0b00000010)&&!(PINB&0b00000001)


/*******************************************/
/*** EEPROM-Variablen                    ***/
/*******************************************/
double        ballDiameterEeprom          EEMEM = 0.034; // Balldurchmesser
unsigned char lcdContrastEeprom           EEMEM = 32;    // LCD-Kontrast
unsigned char currentLanguageEeprom       EEMEM = 1;     // Spracheinstellung
unsigned char gameOwnLeadingGoalsEeprom   EEMEM = 1;
unsigned char gameOppLeadingGoalsEeprom   EEMEM = 1;
unsigned char gameVictoryOwnGoalsEeprom   EEMEM = 5;
unsigned char gameVictoryOppGoalsEeprom   EEMEM = 5;
double        gameHandicapMinSpeedEeprom  EEMEM = 1.0;
double        gameHandicapMaxSpeedEeprom  EEMEM = 100.0;
struct top10record {   // All-Time-Records der Schussgeschwindigkeiten
   uint8_t kuerzel[4]; // 3 Zeichen fr Namen, 1x Nullbyte
   double  speed;      // 4 Byte fr Geschwindigkeit in m/s, Initialisieren auf "   " und 0.0
} top10array[10] EEMEM = {
	{ "   ", 0.0 }, { "   ", 0.0 },
	{ "   ", 0.0 }, { "   ", 0.0 },
	{ "   ", 0.0 }, { "   ", 0.0 },
	{ "   ", 0.0 }, { "   ", 0.0 },
	{ "   ", 0.0 }, { "   ", 0.0 }
};

/*******************************************/
/*** Globale Variablen                   ***/
/*******************************************/
// Knopf-Behandlung
unsigned char okBtn;
unsigned char dnBtn;
unsigned char upBtn;

// Timer-Variablen
unsigned char oldPortA, oldPortC, oldPortD;
double        lastMeasuredSpeed    = 0.0;
volatile unsigned int  lastMeasuredTicks    = 0;
unsigned int  currentMaximum       = 0;
double        lastShotSpeeds [4]   = {0,0,0,0};
unsigned int  timerValuesStart[24] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
unsigned int  timerValuesEnd[24]   = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

// Menzustnde
unsigned char currentScreenState  = 0;
unsigned char currentMenuState    = 0;
unsigned char currentMenuStateOld = 0;
unsigned char currentLanguage     = 0;
unsigned char currentContrast     = 0;
double        currentBallDiameter = 0.0;
unsigned char eventSource         = FROM_MAINMENU;

// Spielvariablen
unsigned char gameVictoryOwnGoals;
unsigned char gameVictoryOppGoals;
unsigned char gameOwnLeadingGoals;
unsigned char gameOppLeadingGoals;
double        gameHandicapMinSpeed;
double        gameHandicapMaxSpeed;
unsigned char gameRunning = 0;
unsigned char gameOppGoals;
unsigned char gameOwnGoals;

extern void clearTimerArrays();
extern void clearTimerStructs();

struct tRecord {
  uint16_t timerValue;
  uint8_t  pinA;
  uint8_t  pinC;
  uint8_t  pinD;
} volatile timerValues[50]; //24*2 worst-Case + 2 Sicherheit, spart malloc, 250 Bytes

volatile uint8_t tRecordIndex;
volatile uint8_t syncVar;


/*******************************************/
/*** INTERRUPT-Routinen (ISR)            ***/
/*******************************************/
ISR(PCINT0_vect) {
	// Keine Messung, wenn main() noch nicht fertig.
	if (syncVar==TIMERSYNC_READY_FOR_CALCULATION) { return; }

	if (syncVar==TIMERSYNC_READY_FOR_MEASURE) {
		if (!(TCCR1B&0b00000011)) {
			TCCR1B = 0b00000011; // Timer an
			TCNT1  = 0;
		}

		timerValues[tRecordIndex].timerValue = TCNT1;
		timerValues[tRecordIndex].pinA = PINA;
		timerValues[tRecordIndex].pinC = PINC;
		timerValues[tRecordIndex].pinD = PIND;
		if (tRecordIndex<49) { tRecordIndex++; }
	}
	// sicherstellen, dass die Lichtschranken alle wieder geschlossen sind...
	if (!(PINA | PINC | PIND)) {
		_delay_us(50);
		if (!(PINA | PINC | PIND)) {
			TCCR1B = 0b00000000; // Timer aus
			if (syncVar == TIMERSYNC_TIMER_IS_OVERFLOWN) {
				syncVar = TIMERSYNC_READY_FOR_MEASURE;
				lastMeasuredSpeed = 0;
				lastMeasuredTicks = 0;
				clearTimerArrays();
				clearTimerStructs();
			} else {
				syncVar = TIMERSYNC_READY_FOR_CALCULATION;
			}
		}
	}
}
// Die beiden ISR sind Aliase fr die oben angegebene.
ISR(PCINT2_vect, ISR_ALIASOF(PCINT0_vect)); 
ISR(PCINT3_vect, ISR_ALIASOF(PCINT0_vect)); 

ISR(TIMER1_OVF_vect) {
	// In der ISR muss der Timer ausgeschaltet werden.
	TCCR1B = 0b00000000; // Timer aus
	TCNT1  = 0;
	syncVar = TIMERSYNC_TIMER_IS_OVERFLOWN;
}

/*******************************************/
/*** NON-Standard GLCD-Funktionen        ***/
/*******************************************/
void glcd_drawMenuFrame(PGM_P headerText) {
	glcd_filledrectangle(0, 0, 131, 9, COLOR_WHITE);
	glcd_filledrectangle(2, 11, 131, 11, COLOR_WHITE);
	glcd_linehorizontal (0, 10, 131);
	glcd_linevertical   (0, 11, 31);
	glcd_linevertical   (1, 11, 31);
	strcpy_P(vBuffer, headerText);
	glcd_puts(1, -2, vBuffer, Tahoma10Bold, 0, COLOR_BLACK);
}

void glcd_renderMenu(
	PGM_P menuHeader, 
	PGM_P menuItems, 
	unsigned char menuItemCount, 
	unsigned char dimension, 
	unsigned char menustate, 
	unsigned char old_menustate) {
	unsigned char i;

	// Pfeil malen
	glcd_filledrectangle(3,16,7,17, COLOR_BLACK);
	glcd_linevertical(6,15,18);

	if (menustate==old_menustate) {
		glcd_drawMenuFrame(menuHeader);
		//ms [0-6] + dim*items*currentlanguage
		strcpy_P(vBuffer, menuItems + dimension * (menustate + currentLanguage * menuItemCount));
		glcd_puts(10, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
		if (menustate<(menuItemCount-1)) {
			strcpy_P(vBuffer, menuItems + dimension*(menustate+1 + currentLanguage*menuItemCount));
			glcd_puts(10, 20, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
		}
	} else {
		//ok, Animieren
		if(menustate>old_menustate) {
			//runterscrollen
			for (i=10;i>0;i-=2) {
				glcd_clear();

				strcpy_P(vBuffer, menuItems + dimension*(menustate-1 + currentLanguage*menuItemCount));
				glcd_puts(10, i, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				strcpy_P(vBuffer, menuItems + dimension*(menustate + currentLanguage*menuItemCount));
				glcd_puts(10, i+10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				if (menustate<(menuItemCount-1)) {
					strcpy_P(vBuffer, menuItems + dimension*(menustate+1 + currentLanguage*menuItemCount));
					glcd_puts(10, i+21, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				}
				glcd_drawMenuFrame(menuHeader);
				glcd_repaint();
				_delay_ms(5);
			}
		} else {
			//raufscrollen
			for (i=10;i<20;i+=2) {
				glcd_clear();
				strcpy_P(vBuffer, menuItems + dimension*(menustate + currentLanguage*menuItemCount));
				glcd_puts(10, i-10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				strcpy_P(vBuffer, menuItems + dimension*(menustate+1 + currentLanguage*menuItemCount));
				glcd_puts(10, i, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				if (menustate<(menuItemCount-2)) {
					strcpy_P(vBuffer, menuItems + dimension*(menustate+2 + currentLanguage*menuItemCount));
					glcd_puts(10, i+11, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				}
				glcd_drawMenuFrame(menuHeader);
				glcd_repaint();
				_delay_ms(5);
			}
		}
	}
}        


/*******************************************/
/*** Kleinere Helperfunktionen           ***/
/*******************************************/
void clearTimerArrays() {
	unsigned char i;
	cli();
	for (i=0;i<24;i++) {
		timerValuesStart[i] = 0;
		timerValuesEnd[i]   = 0;
	}
	lastMeasuredSpeed = 0.0;
	lastMeasuredTicks = 0;
	currentMaximum    = 0;
	sei();
}

void clearTimerStructs() {
	unsigned char i;
	cli();
	for (i=0;i<50;i++) {
		timerValues[i].timerValue = 0;
		timerValues[i].pinA = 0;
		timerValues[i].pinC = 0;
		timerValues[i].pinD = 0;
	}
	tRecordIndex = 0;
	sei();
}


signed char strpos_P(char *haystack, PGM_P needle) {
   char *p = strstr_P(haystack, needle);
   if (p) { return p - haystack; }
   return -1;   // Not found = -1.
}


unsigned char isTop10Record(double *valueToTest) {
	unsigned char i;
	double         tmp;
	if (*valueToTest<10.0) { return 254; }
	for (i=0;i<10;i++) {
		eeprom_read_block(&tmp, &top10array[i].speed, 4);
		if (tmp<*valueToTest) { return i; }
		if (tmp <=0) { break; }
	}
	return 255;
}

void newGame() {
	unsigned char i;
	for (i=0;i<4;i++) { lastShotSpeeds[i] = 0; }
	clearTimerArrays();
	clearTimerStructs();
	syncVar=0;
	lastMeasuredSpeed = 0.0;
	lastMeasuredTicks = 0;
	gameOppGoals = 0;
	gameOwnGoals = 0;
	gameRunning  = 1;
	currentScreenState  = MAINSCREEN;
}


/*******************************************/
/*** START DES HAUPTPROGRAMMS            ***/
/*******************************************/

int main() {
	unsigned char i = 0;
	signed int    currentMenuTempVarInt     = 0;
	unsigned char currentMenuTempVarChar    = 0;
	unsigned char currentMenuTempVarChar2   = 0;
	double        currentMenuTempVardouble  = 0;
	double        currentMenuTempVardouble2 = 0;

	/**************************************/
	/*** MCU Peripherie initialisieren  ***/
	/**************************************/

	// Interrupts fr Lichtschranken
	DDRA  = 0b00000000; // Alles inputs
	PORTA = 0b11111111; // PULLUPS an

	// Serielle Schnittstelle und Taster.
	DDRB  = 0b11111000;
	PORTB = 0b00000111;

	// Interrupts fr Lichtschranken
	DDRC  = 0b00000000; // Alles inputs
	PORTC = 0b11111111; // PULLUPS an

	// Interrupts fr Lichtschranken
	DDRD  = 0b00000000; // Alles inputs
	PORTD = 0b11111111; // PULLUPS an

	// PinChange Interrups konfigurieren
	EICRA = 0b00000000; // Tasten werden gepollt, die serielle Schnittstelle konfiguriert die Pins selbst
	EIMSK = 0b00000000;

	PCICR = 0b00001101;  // PCINT-Gruppen 3,2 und 0 enabled. 1 nicht, da fr SPI bentigt.
	PCMSK3 = 0b11111111;
	PCMSK2 = 0b11111111;
	PCMSK1 = 0b00000000;
	PCMSK0 = 0b11111111;
	

	// TIMER 1 (16 bit konfigurieren)
	TCCR1A = 0b00000000; // Normal Operation, OCR1 Outputs disconnected
	TCCR1B = 0b00000000; // Timer aus (0b00000101 fr F_CPU/1024 als Prescaler)
	TCCR1C = 0b00000000; // Kein Force Prescaler
	TCNT1  = 0; //Timer auf 0 setzen.

	TIMSK1 = 0b00000001; // TC1: Overflow-Interrupt einschalten

	// UART INIT
	SW_UART_Enable();

	// SPI konfigurieren fr DOGM-Interaktion
	SPCR=0b01010000;
	SPSR=0x00;

	/**************************************/
	/*** LCD initialisieren             ***/
	/**************************************/
	// Kommando-Sequenz aus dem Datenblatt
	glcd_hardware_reset();
	glcd_prepare_command();
	spi(0x40);
	spi(0xA1);
	spi(0xC0);
	spi(0xA6);
	spi(0xA2);
	spi(0x2F);
	spi(0xF8);
	spi(0x00);
	spi(0x23);
	spi(0x81);
	spi(0x1F);
	spi(0xAC);
	spi(0x00);
	spi(0xAF);

	/**************************************/
	/*** Splashscreen                   ***/
	/**************************************/
	for (currentMenuTempVarInt=30; currentMenuTempVarInt>-45; currentMenuTempVarInt--) {
		glcd_clear();
		strcpy_P(vBuffer, mainLabel);
		glcd_puts(10, currentMenuTempVarInt,    vBuffer, Tahoma10Bold,    0, COLOR_BLACK);
		strcpy_P(vBuffer, splashAuthor);
		glcd_puts(10, currentMenuTempVarInt+12, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
		strcpy_P(vBuffer, splashUrl);
		glcd_puts(10, currentMenuTempVarInt+24, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
		glcd_repaint();
		_delay_ms(20);
	}

	/**************************************/
	/*** Konfiguration aus EEPROM laden ***/
	/**************************************/
	// Ball Durchmesser laden
	eeprom_read_block(&currentBallDiameter, &ballDiameterEeprom, 4);
	if (currentBallDiameter<0.01 || currentBallDiameter>0.2) {
		currentBallDiameter = 0.034;
		eeprom_write_block(&currentBallDiameter, &ballDiameterEeprom, 4);
	}

	// LCD-Kontrast initialisieren
	currentContrast = eeprom_read_byte(&lcdContrastEeprom);
	if (currentContrast<0 || currentContrast>63) {
		currentContrast = 32;
		eeprom_write_byte(&lcdContrastEeprom, currentContrast);
	}
	glcd_contrast(currentContrast);

	// Sprache laden
	currentLanguage = eeprom_read_byte(&currentLanguageEeprom);
	if (currentLanguage<0 || currentLanguage>1) {
		currentLanguage = 0;
		eeprom_write_byte(&currentLanguageEeprom, currentLanguage);
	}

	// Spielkonfiguration: Fhrende Tore
	gameOwnLeadingGoals = eeprom_read_byte(&gameOwnLeadingGoalsEeprom);
	if (gameOwnLeadingGoals<1 || gameOwnLeadingGoals>10) {
		gameOwnLeadingGoals = 1;
		eeprom_write_byte(&gameOwnLeadingGoalsEeprom, gameOwnLeadingGoals);
	}
	gameOppLeadingGoals = eeprom_read_byte(&gameOppLeadingGoalsEeprom);
	if (gameOppLeadingGoals<1 || gameOppLeadingGoals>10) {
		gameOppLeadingGoals = 1;
		eeprom_write_byte(&gameOppLeadingGoalsEeprom, gameOppLeadingGoals);
	}

	// Spielkonfiguration: Fhrende Tore
	gameVictoryOwnGoals = eeprom_read_byte(&gameVictoryOwnGoalsEeprom);
	if (gameVictoryOwnGoals<0 || gameVictoryOwnGoals>99) {
		gameVictoryOwnGoals = 10;
		eeprom_write_byte(&gameVictoryOwnGoalsEeprom, gameVictoryOwnGoals);
	}
	gameVictoryOppGoals = eeprom_read_byte(&gameVictoryOppGoalsEeprom);
	if (gameVictoryOppGoals<0 || gameVictoryOppGoals>99) {
		gameVictoryOppGoals = 10;
		eeprom_write_byte(&gameVictoryOppGoalsEeprom, gameVictoryOppGoals);
	}

	// Spielkonfiguration: Handicaps laden
	eeprom_read_block(&gameHandicapMinSpeed, &gameHandicapMinSpeedEeprom, 4);
	if (gameHandicapMinSpeed<0.0 || gameHandicapMinSpeed>30.0) {
		gameHandicapMinSpeed = 0.0;
		eeprom_write_block(&gameHandicapMinSpeed, &gameHandicapMinSpeedEeprom, 4);
	}
	eeprom_read_block(&gameHandicapMaxSpeed, &gameHandicapMaxSpeedEeprom, 4);
	if (gameHandicapMaxSpeed<= gameHandicapMinSpeed || gameHandicapMaxSpeed>100.0) {
		gameHandicapMaxSpeed = gameHandicapMinSpeed + 10.0;
		eeprom_write_block(&gameHandicapMaxSpeed, &gameHandicapMaxSpeedEeprom, 4);
	}


	/**************************************/
	/*** Statemachine vorbelegen        ***/
	/**************************************/
	currentScreenState = MAINSCREEN;

	/**************************************/
	/*** Interrupts an                  ***/
	/**************************************/
	sei();

	while (1) {


		/* Tasten behandeln */
		// Fr jede Taste existiert eine entprellte uint8_t Variable.
		// Sie speichert den Zustand: 0=nicht gedrckt, 1=Einmal drcken,
		// 10=lange gedrckt, 50=noch lnger gedrckt
		// Ein schnelles Hochzhlen lt sich deswegen einfach realisieren:
		// (==1 || ==10): Beim ersten Drcken tut man etwas, oder eben wenn
		// die Hauptschleife 10x durchgelaufen ist.
		// Ist leider fr die Variante mit 3Tasten->2Pins etwas umfangreicher
		if (OKBTN) {
			if (okBtn < 50) { okBtn++; }
			dnBtn=0;
			upBtn=0;
		} else if (DNBTN) {
			if (dnBtn < 50) { dnBtn++; }
			okBtn=0;
			upBtn=0;
		} else if (UPBTN) {
			if (upBtn < 50) { upBtn++; }
			okBtn=0;
			dnBtn=0;
		} else {
			okBtn=0;
			dnBtn=0;
			upBtn=0;
		}

		// Hier behandeln wir die seriell empfangenen Daten...
		if (currentScreenState != MAINSCREEN_DEBUG_RXBUFFER) {
			/* NEUES SPIEL ANGEFORDERT */
			if (strpos_P(rxBuffer, newGameRequest)!=-1) {
				clearUsartRxBuffer();
				eventSource        = FROM_USART;
				currentScreenState = MAINMENU_NEWGAME_CONFIRM;
			}
			/* WIR HABEN ES IM GEGNERISCHEN TOR KRACHEN LASSEN */
			if (strpos_P(rxBuffer, goalOpponentHit)!=-1) {
				clearUsartRxBuffer();
				if (gameOwnGoals<99) { 
					gameOwnGoals++;
					currentMenuTempVarInt = 131;
					currentScreenState = MAINSCREEN_ANIMATE_GOAL;
				}
			}
			/* GEGNER HAT SEIN EIGENES TOR ZURCKGENOMMEN */
			if (strpos_P(rxBuffer, goalCanceled)!=-1) {
				clearUsartRxBuffer();
				if (gameOppGoals>0) { gameOppGoals--; }
				currentScreenState  = MAINSCREEN;
			}
			/* GEGNER HAT UNS EIN TOR GUTGESCHRIEBEN */
			if (strpos_P(rxBuffer, goalManuallyAdded)!=-1) {
				clearUsartRxBuffer();
				if (gameOwnGoals<99) { gameOwnGoals++; }
				currentScreenState  = MAINSCREEN;
			}

			/* WIR HABEN DIE SIEGBEDINGUNGEN EMPFANGEN */
			if (strpos_P(rxBuffer, sendVictoryConfig)!=-1 && rxBufferIndex>8) {
				gameVictoryOwnGoals = rxBuffer[4];
				eeprom_write_byte(&gameVictoryOwnGoalsEeprom, gameVictoryOwnGoals);
				gameVictoryOppGoals = rxBuffer[5];
				eeprom_write_byte(&gameVictoryOppGoalsEeprom, gameVictoryOppGoals);
				gameOwnLeadingGoals = rxBuffer[6];;
				eeprom_write_byte(&gameOwnLeadingGoalsEeprom, gameOwnLeadingGoals);
				gameOppLeadingGoals = rxBuffer[7];
				eeprom_write_byte(&gameOppLeadingGoalsEeprom, gameOppLeadingGoals);
				glcd_clear();
				glcd_rectangle(0,0,131,31);
				strcpy_P(vBuffer, dataReceivedLabel[currentLanguage]);
				glcd_puts(10, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				glcd_repaint();
				_delay_ms(1000);
				clearUsartRxBuffer();
				newGame();
			}

			/* WIR HABEN DIE HANDICAPS EMPFANGEN */
			if (strpos_P(rxBuffer, sendHandicapConfigMinSpeed)!=-1 && rxBufferIndex>8) {
				glcd_clear();
				glcd_rectangle(0,0,131,31);
				strcpy_P(vBuffer, dataReceivedLabel[currentLanguage]);
				glcd_puts(10, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				glcd_repaint();
				memcpy(&gameHandicapMinSpeed, &rxBuffer[4], 4);
				eeprom_write_block(&gameHandicapMinSpeed, &gameHandicapMinSpeedEeprom, 4);
				clearUsartRxBuffer();
				_delay_ms(500);
			}
			if (strpos_P(rxBuffer, sendHandicapConfigMaxSpeed)!=-1 && rxBufferIndex>8) {
				glcd_clear();
				glcd_rectangle(0,0,131,31);
				strcpy_P(vBuffer, dataReceivedLabel[currentLanguage]);
				glcd_puts(10, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				glcd_repaint();
				memcpy(&gameHandicapMaxSpeed, &rxBuffer[4], 4);
				eeprom_write_block(&gameHandicapMaxSpeed, &gameHandicapMaxSpeedEeprom, 4);
				clearUsartRxBuffer();
				_delay_ms(500);
			}
		}


		// Jetzt den LCD lschen
		// Dann knnen die einzelnen Screens ihre Inhalte einfach losschreiben.		
		glcd_clear();


		switch (currentScreenState) {

			/*************************************/
			/*** MAINSCREEN                    ***/
			/*************************************/
			case MAINSCREEN:
				if (gameRunning) {
					sprintf(vBuffer, "%02u:%02u", gameOwnGoals, gameOppGoals); // Anzahl der Tore ausgeben.
					glcd_puts(3, 3, vBuffer, ArialBig, 2, COLOR_BLACK);
					for (i=0;i<4;i++) {
						sprintf(vBuffer, "%06.3F", lastShotSpeeds[i]); // Anzahl der Tore ausgeben.
						glcd_puts(100, (i*7)+3, vBuffer, SansSerif7RegularNumbers, 0, COLOR_BLACK);
					}
					glcd_rectangle(0,0,131,31);
					glcd_linevertical(98,0,31);

					// Ok, eine vollstndige Messung von den
					// Interrupt-Betriebenen Lichtschranken
					if (syncVar==TIMERSYNC_READY_FOR_CALCULATION) {
						clearTimerArrays();

						// Erst einmal die Zeiten berechnen
						oldPortA = 0;
						oldPortC = 0;
						oldPortD = 0;
						for (i=0;i<50;i++) {
							if (  (timerValues[i].pinA&0b00000001) && !(oldPortA&0b00000001) && timerValuesStart[18] == 0) { timerValuesStart[18] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinA&0b00000001) &&  (oldPortA&0b00000001) && timerValuesEnd[18] == 0)   { timerValuesEnd[18]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinA&0b00000010) && !(oldPortA&0b00000010) && timerValuesStart[19] == 0) { timerValuesStart[19] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinA&0b00000010) &&  (oldPortA&0b00000010) && timerValuesEnd[19] == 0)   { timerValuesEnd[19]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinA&0b00000100) && !(oldPortA&0b00000100) && timerValuesStart[17] == 0) { timerValuesStart[17] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinA&0b00000100) &&  (oldPortA&0b00000100) && timerValuesEnd[17] == 0)   { timerValuesEnd[17]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinA&0b00001000) && !(oldPortA&0b00001000) && timerValuesStart[20] == 0) { timerValuesStart[20] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinA&0b00001000) &&  (oldPortA&0b00001000) && timerValuesEnd[20] == 0)   { timerValuesEnd[20]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinA&0b00010000) && !(oldPortA&0b00010000) && timerValuesStart[16] == 0) { timerValuesStart[16] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinA&0b00010000) &&  (oldPortA&0b00010000) && timerValuesEnd[16] == 0)   { timerValuesEnd[16]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinA&0b00100000) && !(oldPortA&0b00100000) && timerValuesStart[21] == 0) { timerValuesStart[21] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinA&0b00100000) &&  (oldPortA&0b00100000) && timerValuesEnd[21] == 0)   { timerValuesEnd[21]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinA&0b01000000) && !(oldPortA&0b01000000) && timerValuesStart[15] == 0) { timerValuesStart[15] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinA&0b01000000) &&  (oldPortA&0b01000000) && timerValuesEnd[15] == 0)   { timerValuesEnd[15]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinA&0b10000000) && !(oldPortA&0b10000000) && timerValuesStart[22] == 0) { timerValuesStart[22] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinA&0b10000000) &&  (oldPortA&0b10000000) && timerValuesEnd[22] == 0)   { timerValuesEnd[22]   = timerValues[i].timerValue; } 

							if (  (timerValues[i].pinC&0b00000001) && !(oldPortC&0b00000001) && timerValuesStart[9] == 0) { timerValuesStart[9] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinC&0b00000001) &&  (oldPortC&0b00000001) && timerValuesEnd[9] == 0 ) { timerValuesEnd[9]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinC&0b00000010) && !(oldPortC&0b00000010) && timerValuesStart[0] == 0 ) { timerValuesStart[0] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinC&0b00000010) &&  (oldPortC&0b00000010) && timerValuesEnd[0] == 0 ) { timerValuesEnd[0]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinC&0b00000100) && !(oldPortC&0b00000100) && timerValuesStart[10] == 0 ) { timerValuesStart[10] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinC&0b00000100) &&  (oldPortC&0b00000100) && timerValuesEnd[10] == 0 ) { timerValuesEnd[10]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinC&0b00001000) && !(oldPortC&0b00001000) && timerValuesStart[11] == 0 ) { timerValuesStart[11] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinC&0b00001000) &&  (oldPortC&0b00001000) && timerValuesEnd[11] == 0 ) { timerValuesEnd[11]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinC&0b00010000) && !(oldPortC&0b00010000) && timerValuesStart[12] == 0 ) { timerValuesStart[12] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinC&0b00010000) &&  (oldPortC&0b00010000) && timerValuesEnd[12] == 0 ) { timerValuesEnd[12]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinC&0b00100000) && !(oldPortC&0b00100000) && timerValuesStart[13] == 0 ) { timerValuesStart[13] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinC&0b00100000) &&  (oldPortC&0b00100000) && timerValuesEnd[13] == 0 ) { timerValuesEnd[13]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinC&0b01000000) && !(oldPortC&0b01000000) && timerValuesStart[23] == 0 ) { timerValuesStart[23] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinC&0b01000000) &&  (oldPortC&0b01000000) && timerValuesEnd[23] == 0 ) { timerValuesEnd[23]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinC&0b10000000) && !(oldPortC&0b10000000) && timerValuesStart[14] == 0 ) { timerValuesStart[14] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinC&0b10000000) &&  (oldPortC&0b10000000) && timerValuesEnd[14] == 0 ) { timerValuesEnd[14]   = timerValues[i].timerValue; } 

							if (  (timerValues[i].pinD&0b00000001) && !(oldPortD&0b00000001) && timerValuesStart[5] == 0 ) { timerValuesStart[5] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinD&0b00000001) &&  (oldPortD&0b00000001) && timerValuesEnd[5] == 0 ) { timerValuesEnd[5]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinD&0b00000010) && !(oldPortD&0b00000010) && timerValuesStart[4] == 0 ) { timerValuesStart[4] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinD&0b00000010) &&  (oldPortD&0b00000010) && timerValuesEnd[4] == 0 ) { timerValuesEnd[4]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinD&0b00000100) && !(oldPortD&0b00000100) && timerValuesStart[6] == 0 ) { timerValuesStart[6] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinD&0b00000100) &&  (oldPortD&0b00000100) && timerValuesEnd[6] == 0 ) { timerValuesEnd[6]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinD&0b00001000) && !(oldPortD&0b00001000) && timerValuesStart[3] == 0 ) { timerValuesStart[3] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinD&0b00001000) &&  (oldPortD&0b00001000) && timerValuesEnd[3] == 0 ) { timerValuesEnd[3]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinD&0b00010000) && !(oldPortD&0b00010000) && timerValuesStart[7] == 0 ) { timerValuesStart[7] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinD&0b00010000) &&  (oldPortD&0b00010000) && timerValuesEnd[7] == 0 ) { timerValuesEnd[7]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinD&0b00100000) && !(oldPortD&0b00100000) && timerValuesStart[2] == 0 ) { timerValuesStart[2] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinD&0b00100000) &&  (oldPortD&0b00100000) && timerValuesEnd[2] == 0 ) { timerValuesEnd[2]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinD&0b01000000) && !(oldPortD&0b01000000) && timerValuesStart[8] == 0 ) { timerValuesStart[8] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinD&0b01000000) &&  (oldPortD&0b01000000) && timerValuesEnd[8] == 0 ) { timerValuesEnd[8]   = timerValues[i].timerValue; } 
							if (  (timerValues[i].pinD&0b10000000) && !(oldPortD&0b10000000) && timerValuesStart[1] == 0 ) { timerValuesStart[1] = timerValues[i].timerValue; } 
							if ( !(timerValues[i].pinD&0b10000000) &&  (oldPortD&0b10000000) && timerValuesEnd[1] == 0 ) { timerValuesEnd[1]   = timerValues[i].timerValue; } 

							oldPortA = timerValues[i].pinA;
							oldPortC = timerValues[i].pinC;
							oldPortD = timerValues[i].pinD;

						}

						// jetzt das Maximum suchen
						for (i=0;i<24;i++) {
							if (timerValuesEnd[i]>timerValuesStart[i]) { currentMaximum = timerValuesEnd[i] - timerValuesStart[i]; }
							if (currentMaximum>lastMeasuredTicks) { lastMeasuredTicks = currentMaximum; }
						}

						// jetzt die Arrays clearen
						clearTimerStructs();

						// Wir haben einen Treffer kassiert.
						// mal schauen, wie schlimm es wirklich wird...

						// Ausrechnen der Geschwindigkeit
						lastMeasuredSpeed = (currentBallDiameter * 3.6)/(0.0000064 * ((double)lastMeasuredTicks));

						// 1. Handycaps prfen
						if (lastMeasuredSpeed>=gameHandicapMinSpeed && lastMeasuredSpeed<=gameHandicapMaxSpeed) {
							// 2. Gstetore erhhen
							gameOppGoals++;

							// 3. "Treffer" an Remote senden
							strcpy_P(txBuffer, goalOpponentHit);
							uart_puts(txBuffer);
						
							// 4. Gemessenen Speed hier anzeigen
							glcd_filledrectangle(1,1,130,30,COLOR_WHITE);
							// Das "/" steht im ArialBig Zeichendatz fr das Zeichen "km/h"
							sprintf(vBuffer, "%2.3F/", lastMeasuredSpeed);
							glcd_puts(3, 3, vBuffer, ArialBig, 0, COLOR_BLACK);
							glcd_repaint();

							_delay_ms(GAME_DISPLAY_MESSAGES_DELAY);

							// Historie der High-Scores (letzten 4 Schsse) aktualisieren
							for (i=3;i>0;i--) { lastShotSpeeds[i] = lastShotSpeeds[i-1]; }
							lastShotSpeeds[0] = lastMeasuredSpeed;

							// Testen ob TOP10-wrdig.
							currentMenuTempVarChar = isTop10Record(&lastMeasuredSpeed);
							if (currentMenuTempVarChar<10) {
								// Aha neuer Record.
								// in currentMenuTempVarChar steht drin, an welcher Stelle der Record rein muss
								// currentMenuTempVarChar = INSERT POSITION
								currentMenuTempVarChar2 = 0; // Buchstabenposition
								currentScreenState      = MAINSCREEN_ENTER_TOP10;
								strcpy(txBuffer, "---");
								currentMenuTempVarInt   = txBuffer[currentMenuTempVarChar2];
							}

						} else {
							// Leider am Handicap gescheitert.
							glcd_clear();
					        glcd_drawMenuFrame(errorHandicapNoGoal[currentLanguage]);
							
							if (lastMeasuredSpeed<gameHandicapMinSpeed) {
								strcpy_P(vBuffer, errorHandicapSpeedTooLow[currentLanguage]);
								glcd_puts(10, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
								sprintf(vBuffer, "hc=%6.3F", gameHandicapMinSpeed);
							}
							if (lastMeasuredSpeed>gameHandicapMaxSpeed) {
								strcpy_P(vBuffer, errorHandicapSpeedTooHigh[currentLanguage]);
								glcd_puts(10, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
								sprintf(vBuffer, "hc=%6.3F", gameHandicapMaxSpeed);
							}

							glcd_puts(70, 21, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

							sprintf(vBuffer, "v=%6.3F", lastMeasuredSpeed);
							glcd_puts(10, 21, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

							glcd_repaint();
							_delay_ms(GAME_DISPLAY_MESSAGES_DELAY);

						}
						// neue Messung ermglichen
						syncVar = TIMERSYNC_READY_FOR_MEASURE;
					}

					// Siegbedingungen prfen
					if ( (gameOwnGoals>=gameVictoryOwnGoals) && (gameOwnGoals-gameOppGoals >= gameOwnLeadingGoals) ) {
						currentMenuTempVarInt = 131;
						gameRunning           = 0;
						currentScreenState    = MAINSCREEN_ANIMATE_VICTORY;
					}

					// Verlustbedingung prfen
					if ( (gameOppGoals>=gameVictoryOppGoals) && (gameOppGoals-gameOwnGoals >= gameOppLeadingGoals) ) {
						currentMenuTempVarInt = 131;
						gameRunning           = 0;
						currentScreenState    = MAINSCREEN_ANIMATE_DEFEAT;
					}


					// Buttons...
					if (upBtn==1) {
						eventSource         = FROM_GAMERUNNING;
						currentScreenState  = MAINMENU_COUNTGOAL;
					}
					if (dnBtn==1) {
						eventSource         = FROM_GAMERUNNING;
						currentScreenState  = MAINMENU_CANCELGOAL;
					}
					if (okBtn==1) {
						currentScreenState  = MAINMENU;
						currentMenuState    = 2; // Debuggen Tordaten
						currentMenuStateOld = 2;
					}

				} else {
					glcd_drawMenuFrame(mainLabel);
					strcpy_P(vBuffer, mainLabelDescription[currentLanguage]);
					glcd_puts(4, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
					if (++i>128) {
						sprintf(vBuffer, "P1: vg=%02u:%02u lg=%02u:%02u", gameVictoryOwnGoals, gameVictoryOppGoals, gameOwnLeadingGoals, gameOppLeadingGoals);
					} else {
						sprintf(vBuffer, "P2: hc=%05.3F:%05.3F km/h", gameHandicapMinSpeed, gameHandicapMaxSpeed);
					}
					glcd_puts(4, 20, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				
					if (okBtn==1) {
						currentScreenState  = MAINMENU;
						currentMenuState    = 3;// Zustand "Neues Spiel"
						currentMenuStateOld = 3;
					}
				}



			break;

			case MAINSCREEN_ANIMATE_GOAL:
				glcd_image(currentMenuTempVarInt, 0, goalImage);
				currentMenuTempVarInt-=2;
				if (okBtn==1 || currentMenuTempVarInt < -131 ) {
					currentMenuTempVarInt = 0;
					currentScreenState = MAINSCREEN;
				}
				_delay_ms(10);
			break;

			case MAINSCREEN_ANIMATE_VICTORY:
				sprintf(vBuffer, "%02u:%02u", gameOwnGoals, gameOppGoals); // Anzahl der Tore ausgeben.
				glcd_puts(3, 3, vBuffer, ArialBig, 2, COLOR_BLACK);
				for (i=0;i<4;i++) {
					sprintf(vBuffer, "%06.3F", lastShotSpeeds[i]); // Anzahl der Tore ausgeben.
					glcd_puts(100, (i*7)+3, vBuffer, SansSerif7RegularNumbers, 0, COLOR_BLACK);
				}
				glcd_rectangle(0,0,131,31);
				glcd_linevertical(98,0,31);
				glcd_image(currentMenuTempVarInt, 0, victoryImage);
				currentMenuTempVarInt-=2;
				if (currentMenuTempVarInt < -131 ) { currentMenuTempVarInt = 131; }
				_delay_ms(10);
				lastMeasuredSpeed = 0;
				lastMeasuredTicks = 0;
				gameRunning = 0;
				if (okBtn==1) { currentScreenState  = MAINSCREEN; }
			break;

			case MAINSCREEN_ANIMATE_DEFEAT:
				sprintf(vBuffer, "%02u:%02u", gameOwnGoals, gameOppGoals); // Anzahl der Tore ausgeben.
				glcd_puts(3, 3, vBuffer, ArialBig, 2, COLOR_BLACK);
				for (i=0;i<4;i++) {
					sprintf(vBuffer, "%06.3F", lastShotSpeeds[i]); // Anzahl der Tore ausgeben.
					glcd_puts(100, (i*7)+3, vBuffer, SansSerif7RegularNumbers, 0, COLOR_BLACK);
				}
				glcd_rectangle(0,0,131,31);
				glcd_linevertical(98,0,31);
				glcd_image(currentMenuTempVarInt, 0, defeatImage);
				currentMenuTempVarInt-=2;
				if (currentMenuTempVarInt < -131 ) { currentMenuTempVarInt = 131; }
				_delay_ms(10);
				lastMeasuredSpeed = 0;
				lastMeasuredTicks = 0;
				gameRunning = 0;
				if (okBtn==1) { currentScreenState  = MAINSCREEN; }
			break;

			case MAINSCREEN_DEBUG_RXBUFFER:
		        glcd_drawMenuFrame(debugRxBufferLabel[currentLanguage]);
				sprintf(vBuffer, "%03u %03d %03d %03d %03d %03d %03u", 
					rxBufferIndex,
					SW_UART_status,
					TCCR0A,
					OCR0A,
					TIFR0,
					EIMSK,
					EICRA
				);
				glcd_puts(5, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				glcd_puts(10, 21, rxBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				if (okBtn==1) {
					okBtn=51;
					currentScreenState  = MAINSCREEN;
				}
			break;

			case MAINSCREEN_DEBUG_GOALCOUNTER:
		        glcd_drawMenuFrame(debugGoalCounterLabel[currentLanguage]);

				sprintf(vBuffer, "%05u", lastMeasuredTicks);
				glcd_puts(110, -1, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				sprintf(vBuffer, "%02u %05u:%05u = %05u",
					currentMenuTempVarChar,
					timerValuesStart[currentMenuTempVarChar],
					timerValuesEnd[currentMenuTempVarChar],
					timerValuesEnd[currentMenuTempVarChar] - timerValuesStart[currentMenuTempVarChar]
				);
				glcd_puts(10, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				sprintf(vBuffer, "%02u %05u:%05u = %05u",
					currentMenuTempVarChar+1,
					timerValuesStart[currentMenuTempVarChar+1],
					timerValuesEnd[currentMenuTempVarChar+1],
					timerValuesEnd[currentMenuTempVarChar+1] - timerValuesStart[currentMenuTempVarChar+1]
				);
				glcd_puts(10, 21, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);


				if ((dnBtn==1||dnBtn>10) && currentMenuTempVarChar<22) { currentMenuTempVarChar++; }
				if ((upBtn==1||upBtn>10) && currentMenuTempVarChar>0)  { currentMenuTempVarChar--; }
				if (okBtn==1) { currentScreenState = MAINSCREEN; }
			break;

			case MAINSCREEN_ENTER_TOP10:
		        glcd_drawMenuFrame(enterTop10Label[currentLanguage]);

				glcd_filledrectangle(10, 16, 35, 26, COLOR_BLACK);
				glcd_puts(12, 15, txBuffer, Tahoma10Bold, 0, COLOR_WHITE);
				
				sprintf(vBuffer, "%06.3F km/h", lastMeasuredSpeed);
				glcd_puts(50, 11, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				sprintf(vBuffer, "Position %02u", (currentMenuTempVarChar+1));
				glcd_puts(50, 21, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				// Blinkenden Cursor zeichnen
				if (++i%64>32) { glcd_linehorizontal((currentMenuTempVarChar2*7)+10, 28, (currentMenuTempVarChar2*7)+15); }

				if (okBtn==1) {
					if (currentMenuTempVarChar2==2) {
						// OK, drei Buchstaben eingegeben
						for (i=9;i>currentMenuTempVarChar;i--) {
							// runterschieben der vorigen Eintrge...
							eeprom_read_block(&vBuffer,                   &top10array[i-1].kuerzel, 4);
							eeprom_read_block(&currentMenuTempVardouble,  &top10array[i-1].speed,   4);
							eeprom_write_block(&vBuffer,                  &top10array[i].kuerzel,   4);
							eeprom_write_block(&currentMenuTempVardouble, &top10array[i].speed,     4);
						}
						eeprom_write_block(&lastMeasuredSpeed, &top10array[currentMenuTempVarChar].speed,   4);
						eeprom_write_block(&txBuffer,          &top10array[currentMenuTempVarChar].kuerzel, 4);
						okBtn=11;
						currentScreenState  = MAINSCREEN;
					} else {
						currentMenuTempVarChar2++;
						currentMenuTempVarInt = txBuffer[currentMenuTempVarChar2];
					}
				}
			
				if (upBtn==1 || upBtn>10) {
					currentMenuTempVarInt++;
					if (currentMenuTempVarInt==91) { currentMenuTempVarInt = 32; }
					txBuffer[currentMenuTempVarChar2] = (unsigned char) currentMenuTempVarInt;
				}

				if (dnBtn==1 || dnBtn>10) {
					currentMenuTempVarInt--;
					if (currentMenuTempVarInt==31) { currentMenuTempVarInt = 90; }
					txBuffer[currentMenuTempVarChar2] = (unsigned char) currentMenuTempVarInt;
				}
				
			break;


			/*************************************/
			/*** MAINMENU und seine Items      ***/
			/*************************************/
			case MAINMENU:
		        glcd_renderMenu(mainMenuLabel[currentLanguage], menuItems[0][0], 9, 20, currentMenuState, currentMenuStateOld);
		        currentMenuStateOld = currentMenuState;
		        if ((dnBtn==1||(dnBtn>10&&dnBtn<51))&&currentMenuState<8) { currentMenuState++; }
		        if ((upBtn==1||(upBtn>10&&upBtn<51))&&currentMenuState>0) { currentMenuState--; }

				if (okBtn==1) {
					switch (currentMenuState) {
						case 0:
							eventSource        = FROM_MAINMENU;
							currentScreenState = MAINMENU_CANCELGOAL;
						break;
						case 1:
							eventSource        = FROM_MAINMENU;
							currentScreenState = MAINMENU_COUNTGOAL;
						break;
						case 2:
							eventSource        = FROM_MAINMENU;
							currentScreenState = MAINSCREEN_DEBUG_GOALCOUNTER;
							currentMenuTempVarChar = 0;
						break;
						case 3:
							eventSource        = FROM_MAINMENU;
							currentScreenState = MAINMENU_NEWGAME_CONFIRM;
						break;
						case 4:
							currentScreenState  = VICTORY_MENU;
							currentMenuState    = 0;
							currentMenuStateOld = 0;
						break;
						case 5:
							currentScreenState  = HANDICAP_MENU;
							currentMenuState    = 0;
							currentMenuStateOld = 0;
						break;
						case 6:
							currentScreenState  = CONFIG_MENU;
							currentMenuState    = 0;
							currentMenuStateOld = 0;
						break;
						case 7:
							currentMenuTempVarChar = 0;
							currentMenuTempVarInt  = 31;
							currentScreenState = MAINMENU_TOP10;
						break;
						case 8:
							currentScreenState = MAINSCREEN;
						break;
					}
				}
			break;

			case MAINMENU_NEWGAME_CONFIRM:
		        glcd_drawMenuFrame(menuItems[currentLanguage][3]);
				strcpy_P(vBuffer, confirmNewGameLine1[currentLanguage]);
				glcd_puts(4, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				strcpy_P(vBuffer, dnToCancel[currentLanguage]);
				glcd_puts(4, 20, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				
				// Wenn Gegenseite storniert,
				// machen wir das auch
				if (strpos_P(rxBuffer, newGameCancel)!=-1) {
					clearUsartRxBuffer();
					currentScreenState  = MAINMENU;
				}
				if (okBtn==1) {
					if (eventSource == FROM_MAINMENU) {
						strcpy_P(txBuffer, newGameRequest);
						uart_puts(txBuffer);
						currentScreenState  = MAINMENU_NEWGAME_WAIT_FOR_CONFIRM;
					}
					if (eventSource == FROM_USART) {
						strcpy_P(txBuffer, newGameConfirm);
						uart_puts(txBuffer);
						newGame();
					}
				}
				if (dnBtn==1) {
					if (eventSource == FROM_MAINMENU) {
						currentScreenState  = MAINMENU;
					}
					if (eventSource == FROM_USART) {
						strcpy_P(txBuffer, newGameCancel);
						uart_puts(txBuffer);
						currentScreenState = MAINSCREEN;
					}
					dnBtn = 51;
				}
			break;

			case MAINMENU_NEWGAME_WAIT_FOR_CONFIRM:
		        glcd_drawMenuFrame(menuItems[currentLanguage][3]);
				strcpy_P(vBuffer, waitForConfirmNewGameLine1[currentLanguage]);
				glcd_puts(4, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				strcpy_P(vBuffer, dnToCancel[currentLanguage]);
				glcd_puts(4, 20, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				
				if (strpos_P(rxBuffer, newGameConfirm)!=-1) {
					clearUsartRxBuffer();
					newGame();
				}

				if (strpos_P(rxBuffer, newGameCancel)!=-1) {
					clearUsartRxBuffer();
					currentScreenState  = MAINMENU;
				}
				if (dnBtn==1) {
					strcpy_P(txBuffer, newGameCancel);
					uart_puts(txBuffer);
					currentScreenState  = MAINMENU;
					dnBtn = 51;
				}
			break;

			case MAINMENU_CANCELGOAL:
		        glcd_drawMenuFrame(menuItems[currentLanguage][0]);
				if (gameOwnGoals>0) { strcpy_P(vBuffer, confirmCancelGoalLine1[currentLanguage]); } else { strcpy_P(vBuffer, errorOperationNotPossible[currentLanguage]); }
				glcd_puts(4, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				strcpy_P(vBuffer, dnToCancel[currentLanguage]);
				glcd_puts(4, 20, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				if (okBtn==1 && gameOwnGoals>0) {
					strcpy_P(txBuffer, goalCanceled);
					uart_puts(txBuffer);
					gameOwnGoals--;
					currentScreenState  = MAINSCREEN;
				}
				if (dnBtn==1) {
					if (eventSource == FROM_GAMERUNNING) { currentScreenState  = MAINSCREEN; } else { currentScreenState  = MAINMENU; }
					dnBtn = 51;
				}
			break;

			case MAINMENU_COUNTGOAL:
		        glcd_drawMenuFrame(menuItems[currentLanguage][1]);
				if (gameOppGoals<99) { strcpy_P(vBuffer, confirmCountGoalLine1[currentLanguage]); } else { strcpy_P(vBuffer, errorOperationNotPossible[currentLanguage]); }
				glcd_puts(4, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				strcpy_P(vBuffer, dnToCancel[currentLanguage]);
				glcd_puts(4, 20, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				if (okBtn==1 && gameOppGoals<99) {
					strcpy_P(txBuffer, goalManuallyAdded);
					uart_puts(txBuffer);
					gameOppGoals++;
					currentScreenState  = MAINSCREEN;
				}
				if (dnBtn==1) {
					if (eventSource == FROM_GAMERUNNING) { currentScreenState  = MAINSCREEN; } else { currentScreenState  = MAINMENU; }
					dnBtn = 51;
				}
			break;

			case MAINMENU_TOP10:
				for (i=0;i<10;i++) {
					if (currentMenuTempVarInt+(i*11) < 2 || currentMenuTempVarInt+(i*11) > 30) { continue; }
					eeprom_read_block(&txBuffer, &top10array[i].kuerzel, 4);
					sprintf(vBuffer, "%02u. %s", (i+1), txBuffer);
					glcd_puts( 10, currentMenuTempVarInt+(i*11), vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

					eeprom_read_block(&currentMenuTempVardouble, &top10array[i].speed, 4);
					sprintf(vBuffer, "%06.3F km/h", currentMenuTempVardouble);
					glcd_puts( 60, currentMenuTempVarInt+(i*11), vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				}
				glcd_drawMenuFrame(menuItems[currentLanguage][7]);


				if (++currentMenuTempVarChar == 5) {
					currentMenuTempVarInt--;
					currentMenuTempVarChar = 0;
				}

				if (currentMenuTempVarInt+99 < 2) { currentMenuTempVarInt = 30; }
				if (dnBtn==1 || dnBtn>10) { currentMenuTempVarInt--; }
				if (upBtn==1 || upBtn>10) { currentMenuTempVarInt++; }
				if (okBtn==1) { currentScreenState = MAINMENU; }
			break;



			/*************************************/
			/*** VICTORYMENU und seine Items   ***/
			/*************************************/
			case VICTORY_MENU:
		        glcd_renderMenu(menuItems[currentLanguage][3], menuVictoryItems[0][0], 6, 21, currentMenuState, currentMenuStateOld);
		        currentMenuStateOld = currentMenuState;
		        if ((dnBtn==1||dnBtn>10)&&currentMenuState<5) { currentMenuState++; }
		        if ((upBtn==1||upBtn>10)&&currentMenuState>0) { currentMenuState--; }
				if (okBtn==1) {
					switch (currentMenuState) {
						case 0: currentScreenState = VICTORY_MENU_OWN_GOALS;              break;
						case 1: currentScreenState = VICTORY_MENU_OPPONENT_GOALS;         break;
						case 2: currentScreenState = VICTORY_MENU_OWN_LEADING_GOALS;      break;
						case 3: currentScreenState = VICTORY_MENU_OPPONENT_LEADING_GOALS; break;
						case 4: currentScreenState = VICTORY_MENU_TRANSMIT_CONDITIONS;    break;
						case 5:
							currentScreenState = MAINMENU;
							currentMenuState    = 4; //Zu Handycap springen.
							currentMenuStateOld = 4;
						break;
					}
				}
			break;

			case VICTORY_MENU_OWN_GOALS:
		        glcd_drawMenuFrame(menuVictoryItems[currentLanguage][0]);
	            sprintf(vBuffer, "%02u", gameVictoryOwnGoals);
            	glcd_filledrectangle(19,16,35,26, COLOR_BLACK);
            	glcd_puts(20, 15, vBuffer, Tahoma10Bold, 0, COLOR_WHITE);
				if ((dnBtn==1||dnBtn>10)) { if (gameVictoryOwnGoals>1)  gameVictoryOwnGoals--; else gameVictoryOwnGoals = 99; }
            	if ((upBtn==1||upBtn>10)) { if (gameVictoryOwnGoals<99) gameVictoryOwnGoals++; else gameVictoryOwnGoals = 1;  }
				if (okBtn==1) {
					eeprom_write_byte(&gameVictoryOwnGoalsEeprom, gameVictoryOwnGoals);
					currentScreenState = VICTORY_MENU;
					currentMenuState    = 1;
					currentMenuStateOld = 1;
				}
			break;

			case VICTORY_MENU_OPPONENT_GOALS:
		        glcd_drawMenuFrame(menuVictoryItems[currentLanguage][1]);
	            sprintf(vBuffer, "%02u", gameVictoryOppGoals);
            	glcd_filledrectangle(19,16,35,26, COLOR_BLACK);
            	glcd_puts(20, 15, vBuffer, Tahoma10Bold, 0, COLOR_WHITE);
				if ((dnBtn==1||dnBtn>10)) { if (gameVictoryOppGoals>1)  gameVictoryOppGoals--; else gameVictoryOppGoals = 99; }
            	if ((upBtn==1||upBtn>10)) { if (gameVictoryOppGoals<99) gameVictoryOppGoals++; else gameVictoryOppGoals = 1;  }
				if (okBtn==1) {
					eeprom_write_byte(&gameVictoryOppGoalsEeprom, gameVictoryOppGoals);
					currentScreenState = VICTORY_MENU;
					currentMenuState    = 2;
					currentMenuStateOld = 2;
				}
			break;

			case VICTORY_MENU_OWN_LEADING_GOALS:
		        glcd_drawMenuFrame(menuVictoryItems[currentLanguage][2]);
	            sprintf(vBuffer, "%02u", gameOwnLeadingGoals);
            	glcd_filledrectangle(19,16,35,26, COLOR_BLACK);
            	glcd_puts(20, 15, vBuffer, Tahoma10Bold, 0, COLOR_WHITE);
				if ((dnBtn==1||dnBtn>10)) { if (gameOwnLeadingGoals>1)  gameOwnLeadingGoals--; else gameOwnLeadingGoals = 10; }
            	if ((upBtn==1||upBtn>10)) { if (gameOwnLeadingGoals<10) gameOwnLeadingGoals++; else gameOwnLeadingGoals = 1;  }
				if (okBtn==1) {
					eeprom_write_byte(&gameOwnLeadingGoalsEeprom, gameOwnLeadingGoals);
					currentScreenState = VICTORY_MENU;
					currentMenuState    = 3;
					currentMenuStateOld = 3;
				}
			break;

			case VICTORY_MENU_OPPONENT_LEADING_GOALS:
		        glcd_drawMenuFrame(menuVictoryItems[currentLanguage][3]);
	            sprintf(vBuffer, "%02u", gameOppLeadingGoals);
            	glcd_filledrectangle(19,16,35,26, COLOR_BLACK);
            	glcd_puts(20, 15, vBuffer, Tahoma10Bold, 0, COLOR_WHITE);
				if ((dnBtn==1||dnBtn>10)) { if (gameOppLeadingGoals>1)  gameOppLeadingGoals--; else gameOppLeadingGoals = 10; }
            	if ((upBtn==1||upBtn>10)) { if (gameOppLeadingGoals<10) gameOppLeadingGoals++; else gameOppLeadingGoals = 1;  }
				if (okBtn==1) {
					eeprom_write_byte(&gameOppLeadingGoalsEeprom, gameOppLeadingGoals);
					currentScreenState = VICTORY_MENU;
					currentMenuState    = 4;
					currentMenuStateOld = 4;
				}
			break;

			case VICTORY_MENU_TRANSMIT_CONDITIONS:
		        glcd_drawMenuFrame(menuVictoryItems[currentLanguage][4]);
				strcpy_P(vBuffer, confirmTransmit[currentLanguage]);
				glcd_puts(4, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				strcpy_P(vBuffer, dnToCancel[currentLanguage]);
				glcd_puts(4, 20, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				if (okBtn==1) {
					strcpy_P(txBuffer, sendVictoryConfig);
					// Indizes vertauschen.
					// Die eigenen Tore sollen im Remote-Device als Gegner-Tore gepsiechert werden.
					txBuffer[5] = gameVictoryOwnGoals;
					txBuffer[4] = gameVictoryOppGoals;
					txBuffer[7] = gameOwnLeadingGoals;
					txBuffer[6] = gameOppLeadingGoals; 
					uart_puts(txBuffer);
					okBtn=51;
					newGame();
				}
				if (dnBtn==1) {
					currentScreenState = VICTORY_MENU;
					dnBtn=51;
					currentMenuState    = 4;
					currentMenuStateOld = 4;
				}
			break;

			/*************************************/
			/*** HANDICAP-Men und seine Items ***/
			/*************************************/
			case HANDICAP_MENU:
		        glcd_renderMenu(menuItems[currentLanguage][6], menuHandicapItems[0][0], 4, 23, currentMenuState, currentMenuStateOld);
		        currentMenuStateOld = currentMenuState;
		        if ((dnBtn==1||dnBtn>10)&&currentMenuState<3) { currentMenuState++; }
		        if ((upBtn==1||upBtn>10)&&currentMenuState>0) { currentMenuState--; }

				if (okBtn==1) {
					switch (currentMenuState) {
						case 0: currentScreenState = HANDICAP_MENU_MINSPEED; break;
						case 1: currentScreenState = HANDICAP_MENU_MAXSPEED; break;
						case 2: currentScreenState = HANDICAP_MENU_TRANSMIT_HANDICAPS; break;
						case 3:
							currentScreenState = MAINMENU;
							currentMenuState    = 4;
							currentMenuStateOld = 4;
						break;
					}
				}
			break;

			case HANDICAP_MENU_MINSPEED:
		        glcd_drawMenuFrame(menuHandicapItems[currentLanguage][0]);
				glcd_filledrectangle(18,16,110,26, COLOR_BLACK);
				sprintf(vBuffer, "%05.3F km/h", gameHandicapMinSpeed);
				glcd_puts(20, 15, vBuffer, Tahoma10Bold, 0, COLOR_WHITE);
				if ((upBtn==1||upBtn>10)) {
					if (gameHandicapMinSpeed<20.00) {
						if (upBtn==50) { gameHandicapMinSpeed+=0.1; } else { gameHandicapMinSpeed+=0.01; }
					} else { 
						gameHandicapMinSpeed = 20.00;
					}
				}
				if ((dnBtn==1||dnBtn>10)) {
					if (gameHandicapMinSpeed>0.00) {
						if (dnBtn==50) { gameHandicapMinSpeed-=0.1; } else { gameHandicapMinSpeed-=0.01; }
					} else {
						gameHandicapMinSpeed = 0.0;
					} 
				}
				if (okBtn==1) {
					eeprom_write_block(&gameHandicapMinSpeed, &gameHandicapMinSpeedEeprom, 4);
					currentScreenState = HANDICAP_MENU;
					currentMenuState    = 1;
					currentMenuStateOld = 1;
				}
			break;

			case HANDICAP_MENU_MAXSPEED:
		        glcd_drawMenuFrame(menuHandicapItems[currentLanguage][1]);
				glcd_filledrectangle(18,16,110,26, COLOR_BLACK);
				sprintf(vBuffer, "%05.3F km/h", gameHandicapMaxSpeed);
				glcd_puts(20, 15, vBuffer, Tahoma10Bold, 0, COLOR_WHITE);
				if ((upBtn==1||upBtn>10)) {
					if (gameHandicapMaxSpeed<100.00) {
						if (upBtn==50) { gameHandicapMaxSpeed+=0.1; } else { gameHandicapMaxSpeed+=0.01; }
					} else { 
						gameHandicapMaxSpeed = 100.00;
					}
				}
				if ((dnBtn==1||dnBtn>10)) {
					if (gameHandicapMaxSpeed>gameHandicapMinSpeed+1.0) {
						if (dnBtn==50) { gameHandicapMaxSpeed-=0.1; } else { gameHandicapMaxSpeed-=0.01; }
					} else {
						gameHandicapMaxSpeed = gameHandicapMinSpeed+1.0;
					} 
				}
				if (okBtn==1) {
					eeprom_write_block(&gameHandicapMaxSpeed, &gameHandicapMaxSpeedEeprom, 4);
					currentScreenState = HANDICAP_MENU;
					currentMenuState    = 2;
					currentMenuStateOld = 2;
				}
			break;

			case HANDICAP_MENU_TRANSMIT_HANDICAPS:
		        glcd_drawMenuFrame(menuHandicapItems[currentLanguage][2]);
				strcpy_P(vBuffer, confirmTransmit[currentLanguage]);
				glcd_puts(4, 10, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				strcpy_P(vBuffer, dnToCancel[currentLanguage]);
				glcd_puts(4, 20, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				if (okBtn==1) {
					currentScreenState = HANDICAP_MENU;
					okBtn=51;
					currentMenuState    = 3;
					currentMenuStateOld = 3;

					strcpy_P(txBuffer, sendHandicapConfigMinSpeed);
					memcpy(&txBuffer[4], &gameHandicapMinSpeed, 4);
					txBufferIndex = 8;
					uart_puts(txBuffer);
					_delay_ms(500);

					strcpy_P(txBuffer, sendHandicapConfigMaxSpeed);
					memcpy(&txBuffer[4], &gameHandicapMaxSpeed, 4);
					txBufferIndex = 8;
					uart_puts(txBuffer);
				}
				if (dnBtn==1) {
					currentScreenState = HANDICAP_MENU;
					dnBtn=51;
					currentMenuState    = 2;
					currentMenuStateOld = 2;
				}
			break;



			/*************************************/
			/*** CONFIG-Men und seine Items   ***/
			/*************************************/
			case CONFIG_MENU:
		        glcd_renderMenu(menuItems[currentLanguage][5], menuConfigItems[0][0], 6, 15, currentMenuState, currentMenuStateOld);
		        currentMenuStateOld = currentMenuState;
		        if ((dnBtn==1||dnBtn>10)&&currentMenuState<5) { currentMenuState++; }
		        if ((upBtn==1||upBtn>10)&&currentMenuState>0) { currentMenuState--; }

				if (okBtn==1) {
					switch (currentMenuState) {
						case 0: currentScreenState = CONFIG_MENU_BALLDIAMETER; break;
						case 1:
							currentScreenState = CONFIG_MENU_CALIBRATE;
							currentMenuTempVarChar = 100;
						break;
						case 2: currentScreenState = CONFIG_MENU_TEST_LICHTSCHRANKEN; break;
						case 3: currentScreenState = CONFIG_MENU_LCDLANGUAGE; break;
						case 4: currentScreenState = CONFIG_MENU_LCDCONTRAST; break;
						case 5:
							currentScreenState = MAINMENU;
							currentMenuState = 0;
							currentMenuStateOld = 0;
						break;
					}
				}
			break;

			case CONFIG_MENU_BALLDIAMETER:
		        glcd_drawMenuFrame(menuConfigItems[currentLanguage][0]);
				glcd_filledrectangle(18,16,110,26, COLOR_BLACK);
				sprintf(vBuffer, "%02.2F mm", currentBallDiameter * 1000.0);
				glcd_puts(20, 15, vBuffer, Tahoma10Bold, 0, COLOR_WHITE);
				if ((dnBtn==1||dnBtn>10)) { if (currentBallDiameter>0.02) currentBallDiameter-=0.00001; else currentBallDiameter = 0.02; }
				if ((upBtn==1||upBtn>10)) { if (currentBallDiameter<0.05) currentBallDiameter+=0.00001; else currentBallDiameter = 0.05; }
				if (okBtn==1) {
					eeprom_write_block(&currentBallDiameter, &ballDiameterEeprom, 4);
					currentScreenState = CONFIG_MENU;
				}

			break;

			case CONFIG_MENU_CALIBRATE:
		        glcd_drawMenuFrame(menuConfigItems[currentLanguage][1]);
				glcd_filledrectangle(68,12,105,22, COLOR_BLACK);

				strcpy_P(vBuffer, calibrateHeightLabel[currentLanguage]);
				glcd_puts(10, 11, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);


				sprintf(vBuffer, "%03u cm", currentMenuTempVarChar);
				glcd_puts(70, 11, vBuffer, Tahoma10Regular, 0, COLOR_WHITE);

				if ((dnBtn==1||dnBtn>10)) { if (currentMenuTempVarChar>0)    currentMenuTempVarChar--; else currentMenuTempVarChar = 0; }
				if ((upBtn==1||upBtn>10)) { if (currentMenuTempVarChar<0xFF) currentMenuTempVarChar++; else currentMenuTempVarChar = 0xFF; }
				if (okBtn==1) {
					currentMenuTempVardouble  = 0; // Erfasst die Summe der gemessenen Radien.
					currentMenuTempVardouble2 = 0; // Speichert die letzte gemessene Zeit.
					currentMenuTempVarChar2  = 0; // Speichert die Anzahl der Messungen
					currentScreenState       = CONFIG_MENU_CALIBRATE_MEASURE;
				}
			break;

			case CONFIG_MENU_CALIBRATE_MEASURE:
		        glcd_drawMenuFrame(menuConfigItems[currentLanguage][1]);

				sprintf(vBuffer, "h=%03u cm", currentMenuTempVarChar);
				glcd_puts(70, -2, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				if (lastMeasuredTicks > 0) {
					// AHA, Messung vorgenommen und gltig (mind. 3 Lichtschranken unterbrochen)!
					currentMenuTempVarChar2++;
					currentMenuTempVardouble2 = (double) lastMeasuredTicks;
					currentMenuTempVardouble += sqrt(19.62 * (((double)currentMenuTempVarChar)/100.0)   ) * ((double)lastMeasuredTicks);
					lastMeasuredSpeed = 0;
				}

				sprintf(vBuffer, "k=%02u", currentMenuTempVarChar2); // Anzahl der Versuche ausgeben.
				glcd_puts(10, 11, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				sprintf(vBuffer, "t=%02.4Fms", currentMenuTempVardouble2 * 1000.0);
				glcd_puts(70, 11, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				//sprintf(vBuffer, "%02u:%02u", runningTimersMax, runningTimers); // Anzahl der Versuche ausgeben.
				//glcd_puts(70, 21, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);

				if (currentMenuTempVarChar2>0) {
					sprintf(vBuffer, "d=%02.2F mm", currentMenuTempVardouble / ((double) currentMenuTempVarChar2) * 1000.0 ); // Averaged Durchmesser in mm
					glcd_puts(10, 21, vBuffer, Tahoma10Regular, 0, COLOR_BLACK);
				}

				if (dnBtn==50) {
					// bernehmen und speichern
					eeprom_write_block(&currentMenuTempVardouble, &ballDiameterEeprom, 4);
					currentBallDiameter = currentMenuTempVardouble;
					currentScreenState = CONFIG_MENU;
				}

				if (okBtn==1) {
					// Abbrechen
					currentScreenState = CONFIG_MENU;
				}
			break;



			case CONFIG_MENU_LCDLANGUAGE:
		        glcd_drawMenuFrame(menuConfigItems[currentLanguage][3]);
				glcd_filledrectangle(18,16,110,26, COLOR_BLACK);
				strcpy_P(vBuffer, languageLabel[currentLanguage]);
				glcd_puts(20, 15, vBuffer, Tahoma10Bold, 0, COLOR_WHITE);
				if ((dnBtn==1||dnBtn>10)) { if (currentLanguage>0) currentLanguage--; else currentLanguage = 1; }
				if ((upBtn==1||upBtn>10)) { if (currentLanguage<1) currentLanguage++; else currentLanguage = 0; }
				if (okBtn==1) {
					//Sprache bernehmen
					eeprom_write_byte(&currentLanguageEeprom, currentLanguage);
					currentScreenState = CONFIG_MENU;
				}
			break;

			case CONFIG_MENU_LCDCONTRAST:
		        glcd_drawMenuFrame(menuConfigItems[currentLanguage][4]);
	            sprintf(vBuffer, "%02u", currentContrast);
            	glcd_filledrectangle(19,16,35,26, COLOR_BLACK);
            	glcd_puts(20, 15, vBuffer, Tahoma10Bold, 0, COLOR_WHITE);
            	glcd_contrast(currentContrast);
            	if ((dnBtn==1||dnBtn>10)) { if (currentContrast>0)  currentContrast--; else currentContrast = 63; }
            	if ((upBtn==1||upBtn>10)) { if (currentContrast<63) currentContrast++; else currentContrast = 0;  }
				if (okBtn==1) {
					eeprom_write_byte(&lcdContrastEeprom, currentContrast);
					currentScreenState = CONFIG_MENU;
				}
			break;

			case CONFIG_MENU_TEST_LICHTSCHRANKEN:
				currentMenuTempVarChar=7;
				glcd_drawMenuFrame(menuConfigItems[currentLanguage][2]);

				if (PINA&0b00000001) { timerValuesStart[18] = 1; } else { timerValuesStart[18] = 0; } // 19
				if (PINA&0b00000010) { timerValuesStart[19] = 1; } else { timerValuesStart[19] = 0; } // 20
				if (PINA&0b00000100) { timerValuesStart[17] = 1; } else { timerValuesStart[17] = 0; } // 18
				if (PINA&0b00001000) { timerValuesStart[20] = 1; } else { timerValuesStart[20] = 0; } // 21
				if (PINA&0b00010000) { timerValuesStart[16] = 1; } else { timerValuesStart[16] = 0; } // 17
				if (PINA&0b00100000) { timerValuesStart[21] = 1; } else { timerValuesStart[21] = 0; } // 22
				if (PINA&0b01000000) { timerValuesStart[15] = 1; } else { timerValuesStart[15] = 0; } // 16
				if (PINA&0b10000000) { timerValuesStart[22] = 1; } else { timerValuesStart[22] = 0; } // 23

				if (PINC&0b00000001) { timerValuesStart[9]  = 1; } else { timerValuesStart[9]  = 0; } // 10
				if (PINC&0b00000010) { timerValuesStart[0]  = 1; } else { timerValuesStart[0]  = 0; } //  1
				if (PINC&0b00000100) { timerValuesStart[10] = 1; } else { timerValuesStart[10] = 0; } // 11
				if (PINC&0b00001000) { timerValuesStart[11] = 1; } else { timerValuesStart[11] = 0; } // 12
				if (PINC&0b00010000) { timerValuesStart[12] = 1; } else { timerValuesStart[12] = 0; } // 13
				if (PINC&0b00100000) { timerValuesStart[13] = 1; } else { timerValuesStart[13] = 0; } // 14
				if (PINC&0b01000000) { timerValuesStart[23] = 1; } else { timerValuesStart[23] = 0; } // 24
				if (PINC&0b10000000) { timerValuesStart[14] = 1; } else { timerValuesStart[14] = 0; } // 15

				if (PIND&0b00000001) { timerValuesStart[5]  = 1; } else { timerValuesStart[5]  = 0; }  // 6
				if (PIND&0b00000010) { timerValuesStart[4]  = 1; } else { timerValuesStart[4]  = 0; }  // 5
				if (PIND&0b00000100) { timerValuesStart[6]  = 1; } else { timerValuesStart[6]  = 0; }  // 7
				if (PIND&0b00001000) { timerValuesStart[3]  = 1; } else { timerValuesStart[3]  = 0; }  // 4
				if (PIND&0b00010000) { timerValuesStart[7]  = 1; } else { timerValuesStart[7]  = 0; }  // 8
				if (PIND&0b00100000) { timerValuesStart[2]  = 1; } else { timerValuesStart[2]  = 0; }  // 3
				if (PIND&0b01000000) { timerValuesStart[8]  = 1; } else { timerValuesStart[8]  = 0; }  // 9
				if (PIND&0b10000000) { timerValuesStart[1]  = 1; } else { timerValuesStart[1]  = 0; }  // 2

				for (i=0;i<24;i++) {
					if (timerValuesStart[i] == 0) {
						glcd_filledrectangle(currentMenuTempVarChar,20,currentMenuTempVarChar+3,23,COLOR_BLACK);
					} else {
						glcd_rectangle(currentMenuTempVarChar,20,currentMenuTempVarChar+3,23);
					}
					currentMenuTempVarChar +=5;
				}

				if (okBtn==1) {
					currentScreenState = CONFIG_MENU;
					clearTimerArrays();
				}
			break;
		}		


		glcd_repaint();

	}
}
